#!/usr/bin/env python

"""
An absurdly minimal HTML parser.

Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.utils.htmlparse.token import Tokeniser
from moinformat.utils.htmlparse.tree import Attribute, AttributeValue, \
                                            Comment, Directive, Element, \
                                            Fragment, Inclusion, Node, Text



# Token processing employing the tokens from tokenisation.

class Visitor:
    def __init__(self):
        self.node = Fragment()
        self.stack = [self.node]

    def append(self, node):
        self.node.nodes.append(node)

    def push(self, node):
        self.stack.append(node)
        self.append(node)
        self.node = node

    def pop(self):
        self.stack.pop()
        self.node = self.stack[-1]

    def visit(self, token):
        token.visit(self)

    # Specific handler methods.

    def attribute(self, token):
        if isinstance(self.node, Element):
            self.node.attributes.append(Attribute(token.value))
        else:
            raise ValueError, token

    def attribute_value(self, token):
        if isinstance(self.node, Element):
            self.node.attributes[-1].value = AttributeValue(token.value, token.quote)
        else:
            raise ValueError, token

    def comment(self, token):
        self.append(Comment(token.value))

    def directive(self, token):
        self.append(Directive(token.value))

    def inclusion(self, token):
        self.append(Inclusion(token.value))

    def tag(self, token):
        if not token.is_end():
            self.push(Element(token.tag_name()))
        elif self.node.name == token.tag_name():
            self.pop()
        else:
            raise ValueError, token

    def tag_close(self, token):
        self.pop()

    def text(self, token):
        self.append(Text(token.value))



# Parsing and document construction.

class Parser:
    def __init__(self, text):
        self.tokeniser = Tokeniser(text)
        self.visitor = Visitor()

    def __iter__(self):
        return self

    def next(self):
        token = self.tokeniser.next()
        self.visitor.visit(token)

    def parse(self):
        for _none in self:
            pass

        return self.visitor.node

# vim: tabstop=4 expandtab shiftwidth=4
